# SPL_CGT_INCLUDE: ../../opt/.splpy/common/py_state.cgt
# SPL_CGT_INCLUDE: ../../opt/.splpy/common/py_no_checkpoint_support.cgt

package SelectCustomers_h;
use strict; use Cwd 'realpath';  use File::Basename;  use lib dirname(__FILE__);  use SPL::Operator::Instance::OperatorInstance; use SPL::Operator::Instance::Annotation; use SPL::Operator::Instance::Context; use SPL::Operator::Instance::Expression; use SPL::Operator::Instance::ExpressionTree; use SPL::Operator::Instance::ExpressionTreeEvaluator; use SPL::Operator::Instance::ExpressionTreeVisitor; use SPL::Operator::Instance::ExpressionTreeCppGenVisitor; use SPL::Operator::Instance::InputAttribute; use SPL::Operator::Instance::InputPort; use SPL::Operator::Instance::OutputAttribute; use SPL::Operator::Instance::OutputPort; use SPL::Operator::Instance::Parameter; use SPL::Operator::Instance::StateVariable; use SPL::Operator::Instance::TupleValue; use SPL::Operator::Instance::Window; 
sub main::generate($$) {
   my ($xml, $signature) = @_;  
   print "// $$signature\n";
   my $model = SPL::Operator::Instance::OperatorInstance->new($$xml);
   unshift @INC, dirname ($model->getContext()->getOperatorDirectory()) . "/../impl/nl/include";
   $SPL::CodeGenHelper::verboseMode = $model->getContext()->isVerboseModeOn();
   print '/*', "\n";
   print ' * # Licensed Materials - Property of IBM', "\n";
   print ' * # Copyright IBM Corp. 2017', "\n";
   print ' */', "\n";
   require "splpy_operator.pm";
   my $pyStateful = splpy_OperatorCallable() eq 'class' ? 1 : 0;
   print "\n";
    
    # State handling setup for Python operators.
    # Requires
    #     $pyStateful is set to 0/1 if the operator's callable is not/stateful
    #
    # Sets CPP defines:
    #     SPLPY_OP_STATE_HANDLER - Set to 1 if the operator needs a state handle.
    #     SPLPY_OP_CR - Set to 1 is the operator is in a consistent region
    #     SPLPY_CALLABLE_STATEFUL - Set to 1 if the callable is stateful
    #     SPLPY_CALLABLE_STATE_HANDLER - Set to 1 if op must preserve callable state
   
    my $isWindowed = 0;
    for (my $p = 0; $p < $model->getNumberOfInputPorts(); $p++) {
      if ($model->getInputPortAt($p)->hasWindow()) {
         $isWindowed = 1;
         last;
      }
    }
   
    my $isInConsistentRegion = $model->getContext()->getOptionalContext("ConsistentRegion") ? 1 : 0;
    my $ckptKind = $model->getContext()->getCheckpointingKind();
    my $splpy_op_stateful = ($pyStateful or $isWindowed) && ($isInConsistentRegion or $ckptKind ne "none") ? 1 : 0;
   print "\n";
   print "\n";
   print '#define SPLPY_OP_STATE_HANDLER ';
   print $splpy_op_stateful;
   print "\n";
   print '#define SPLPY_OP_CR ';
   print $isInConsistentRegion;
   print "\n";
   print '#define SPLPY_CALLABLE_STATEFUL ';
   print $pyStateful ? 1 : 0;
   print "\n";
   print '#define SPLPY_CALLABLE_STATE_HANDLER (SPLPY_OP_STATE_HANDLER && SPLPY_CALLABLE_STATEFUL)', "\n";
   print "\n";
   print '#include "splpy.h"', "\n";
   print '#include "splpy_pyop.h"', "\n";
   print "\n";
   print 'using namespace streamsx::topology;', "\n";
   print "\n";
   SPL::CodeGen::headerPrologue($model);
   print "\n";
   print "\n";
   { 
    # Prevent compilation if checkpointing or consistent region are enabled.
   
    my $isInConsistentRegion = $model->getContext()->getOptionalContext("ConsistentRegion");
    my $ckptKind = $model->getContext()->getCheckpointingKind();
   
    my $isCheckpointing = ($isInConsistentRegion or $ckptKind ne "none");
   
    if ($isCheckpointing) {
      # TODO
      # For topology operators, the source location here is unhelpful, since
      # it refers to a location in a generated file that the user generally
      # cannot see.  It would be better to read the @spl_note containing
      # the original python source location and report that.
      SPL::CodeGen::exitln("Checkpointing is not supported for the " . $model->getContext()->getClass() . " operator.", $model->getContext()->getSourceLocation());
    }
   }
   print "\n";
   print "\n";
     my %h_tuple_types;
   print "\n";
   print "\n";
   print 'class MY_OPERATOR : public MY_BASE_OPERATOR, public SplpyPrimitiveOp', "\n";
   print '{', "\n";
   print 'public:', "\n";
   print '  MY_OPERATOR();', "\n";
   print '  virtual ~MY_OPERATOR(); ', "\n";
   print '  void allPortsReady();', "\n";
   print '  void process(uint32_t idx);', "\n";
   print '  void prepareToShutdown(); ', "\n";
   print '  void process(Tuple const & tuple, uint32_t port);', "\n";
   print '  void process(Punctuation const & punct, uint32_t port);', "\n";
   print '  ', "\n";
   print '  void convertAndSubmit(uint32_t port, PyObject *tuple_);', "\n";
   print '  void submitPunct(uint32_t port);', "\n";
   print "\n";
   print 'private:', "\n";
   print '    Mutex _mutex;', "\n";
   print '    SplpyOp *op() { return pyop_; }', "\n";
   print "\n";
   print '    SplpyPyOp *pyop_;', "\n";
   print "\n";
   if ($model->getNumberOfInputPorts() != 0) {
   print "\n";
   print '    PyObject *pyinputfns_;', "\n";
   print '    PyObject *pyInNames_; // port 0', "\n";
     for (my $p = 1; $p < $model->getNumberOfInputPorts(); $p++) { 
   print "\n";
   print '    PyObject *pyInNames_';
   print $p;
   print ';', "\n";
   }}
   print "\n";
   print "\n";
    for (my $p = 0; $p < $model->getNumberOfOutputPorts(); $p++) {
       my $oport = $model->getOutputPortAt($p);
   print "\n";
   print '  PyObject *pyOutNames_';
   print $oport->getIndex();
   print ';', "\n";
   print '  void pySubmitTuplesPort';
   print $oport->getIndex();
   print '(PyObject * value);', "\n";
   print '  void fromPythonToPort';
   print $oport->getIndex();
   print '(PyObject *pyTuple, ';
   print $oport->getCppTupleType();
   print ' &otuple);', "\n";
   print '  void fromPythonDictToPort';
   print $oport->getIndex();
   print '(PyObject *pyDict, ';
   print $oport->getCppTupleType();
   print ' &otuple);', "\n";
   print "\n";
   print '  ';
        my $otype = $oport->getCppTupleType();
        if (! exists $h_tuple_types{$otype}) {
           $h_tuple_types{$otype} = 1;
     
   print "\n";
   print '  void fromPyTupleToSPLTuple(PyObject *pyDict, ';
   print $oport->getCppTupleType();
   print ' & otuple);', "\n";
   print '  void fromPyDictToSPLTuple(PyObject *pyTuple, ';
   print $oport->getCppTupleType();
   print ' & otuple);', "\n";
   print '  ';
   }
   print "\n";
   }
   print "\n";
   print '}; ', "\n";
   print "\n";
   SPL::CodeGen::headerEpilogue($model);
   print "\n";
   CORE::exit $SPL::CodeGen::USER_ERROR if ($SPL::CodeGen::sawError);
}
1;
